package com.supervision.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.fasterxml.jackson.core.type.TypeReference;
import com.supervision.config.domain.GlobalResult;
import com.supervision.domain.IrKnowledge;
import com.supervision.domain.IrKnowledgeSimilar;
import com.supervision.dto.MatchQuestionAnswerDTO;
import com.supervision.dto.MatchToolQuestionDTO;
import com.supervision.dto.QuestionReqDTO;
import com.supervision.exception.BusinessException;
import com.supervision.service.IrKnowledgeService;
import com.supervision.service.IrKnowledgeSimilarService;
import com.supervision.service.MatchToolService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.lang.reflect.Type;
import java.net.HttpURLConnection;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class MatchToolServiceImpl implements MatchToolService {

    private final IrKnowledgeService irKnowledgeService;

    private final IrKnowledgeSimilarService irKnowledgeSimilarService;

    @Value("${matchTool.url}")
    private String matchToolUrl;

    @Override
    public void refreshMatchToolLibrary() {
        // 获取所有的问题
        List<IrKnowledge> knowledgeList = irKnowledgeService.list();
        // 获取相似问题
        List<IrKnowledgeSimilar> similarList = irKnowledgeSimilarService.list();
        Map<String, List<IrKnowledgeSimilar>> similarKnowledgeMap = similarList.stream().collect(Collectors.groupingBy(IrKnowledgeSimilar::getKnowledgeId));
        Map<String, List<String>> questionLibrary = knowledgeList.stream().collect(Collectors.toMap(IrKnowledge::getId, e -> CollUtil.newArrayList(e.getStandardQuestion())));
        List<MatchToolQuestionDTO> matchToolQuestionDTOList = new ArrayList<>();

        for (Map.Entry<String, List<String>> entry : questionLibrary.entrySet()) {
            String questionId = entry.getKey();
            List<IrKnowledgeSimilar> similarKnowledgeList = similarKnowledgeMap.get(questionId);
            if (CollUtil.isNotEmpty(similarKnowledgeList)) {
                Set<String> similarQuestionSet = similarKnowledgeList.stream().map(IrKnowledgeSimilar::getSimilarQuestion).collect(Collectors.toSet());
                entry.getValue().addAll(similarQuestionSet);
            }
            MatchToolQuestionDTO matchToolQuestionDTO = new MatchToolQuestionDTO();
            matchToolQuestionDTO.setQuestionId(questionId);
            matchToolQuestionDTO.setQuestionList(entry.getValue());
            matchToolQuestionDTOList.add(matchToolQuestionDTO);
        }
        // 调用tool服务进行更新操作
        submitRefresh(matchToolQuestionDTOList);
    }

    @Override
    public List<MatchQuestionAnswerDTO> execMatch(String question) {
        log.info("开始调用talkQaSimilarity,问题:{}", question);
        try {
            String post = HttpUtil.post(matchToolUrl + "/matchQuestion", JSONUtil.toJsonStr(new QuestionReqDTO(question)));
            log.info("相似度匹配答案:{}", post);
            TypeReference<GlobalResult<List<MatchQuestionAnswerDTO>>> globalResultTypeReference = new TypeReference<GlobalResult<List<MatchQuestionAnswerDTO>>>() {
            };
            GlobalResult<List<MatchQuestionAnswerDTO>> result = JSONUtil.toBean(post, globalResultTypeReference.getType(), true);
            if (result.getCode() != 200) {
                throw new BusinessException("匹配失败");
            }

            return result.getData();
        } catch (Exception e) {
            log.error("调用talkQaSimilarity error ", e);
            return null;
        }
    }

    private void submitRefresh(List<MatchToolQuestionDTO> matchToolQuestionDTOList) {
        log.info("开始调用matchTool服务进行更新操作");
        try {
            String post = HttpUtil.post(matchToolUrl + "/updateDatabase", JSONUtil.toJsonStr(matchToolQuestionDTOList));
            log.info(post);
            if (!JSONUtil.isTypeJSON(post)) {
                throw new BusinessException("更新失败");
            }
        } catch (Exception e) {
            log.error("调用matchTool服务更新失败", e);
            throw new BusinessException("更新失败");
        }
        log.info("成功调用matchTool服务更新数据库");


    }


}
